float4x4 matWorldViewProj;    // World * View * Projection matrix
float4x4 matWorld;            // world matrix
float3 lightDir;              // light direction
float3 cameraPos;             // camera position



struct VS_OUTPUT 
{
    float4 Pos   : POSITION;   
    float2 Tex : TEXCOORD0;
    float3 Light : TEXCOORD1;
    float3 View : TEXCOORD2;
};

VS_OUTPUT BumpVS( float4 vPos : POSITION, float2 Tex : TEXCOORD, float3 Norm : NORMAL,  float3 Tangent : TANGENT)
{
    VS_OUTPUT Output;
    
    Output.Pos = mul(vPos, matWorldViewProj);
    
    
    float3x3 worldToTangentSpace;
    worldToTangentSpace[0] = mul(Tangent, matWorld);
    worldToTangentSpace[1] = mul(cross(Tangent,Norm), matWorld);
    worldToTangentSpace[2] = mul(Norm, matWorld);
    
    Output.Tex = Tex;
    
    float4 posWorld = mul(vPos, matWorld);
    Output.Light = mul(worldToTangentSpace, lightDir);
    Output.View = mul(worldToTangentSpace, cameraPos - posWorld);

    return Output;    
}

    
struct PS_OUTPUT
{
    float4 Col : COLOR0;
};


texture MainTexture;
sampler MainTextureSampler = sampler_state
{
  Texture = <MainTexture>;
  MinFilter = Linear;
  MagFilter = Linear;
  MipFilter = Linear;
};

texture BumpMap;
sampler BumpMapSampler = sampler_state
{
  Texture = <BumpMap>;
  MinFilter = Linear;
  MagFilter = Linear;
  MipFilter = Linear;
};


PS_OUTPUT BumpPS(float2 TexCoord : TEXCOORD0, float3 Light : TEXCOORD1, float3 View : TEXCOORD2) 
{ 
    PS_OUTPUT Output;
    Output.Col = float4(0,0,0,0);
    
    // fetch color
    float4 color = tex2D(MainTextureSampler, TexCoord);

    float3 normal = (2 * (tex2D(BumpMapSampler, TexCoord))) - 1.0;

    float3 lightDir = normalize(Light);
    float3 viewDir = normalize(View);
    
    // diffuse
    float diffuse = saturate(dot(normal, lightDir));
    
    // reflection vector
    float3 reflect = normalize(2 * diffuse * normal - lightDir);
    
    // specular
    float specular = pow(saturate(dot(reflect, viewDir)), 3);
    
    Output.Col = 0.2 * color + color * diffuse + specular;
    
    return Output;
}

texture BumpHeight;
sampler HeightMapSampler = sampler_state
{
   Texture = <BumpHeight>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
};

float4 ParallaxPS(float2 Tex: TEXCOORD0, float3 Light : TEXCOORD1, float3 View : TEXCOORD2): COLOR
{
   const float scale = {0.04f};
   const float bias = {0.02};

   float3 LightDir = normalize(Light);
   float3 ViewDir = normalize(View);
   float4 Color;
   float3 Normal; 
   
   // fetch height map
   float Height = scale * tex2D(HeightMapSampler, Tex) - bias; 
   
   // compute new texture coordinates
   float2 TexCorrected  = Height * ViewDir + Tex; 
   
   // fetch color map
   Color = tex2D(MainTextureSampler, TexCorrected); 
      
   // bump map
   Normal =(2 * (tex2D(BumpMapSampler, TexCorrected )))- 1.0; 
   
   
   // diffuse
    float diffuse = saturate(dot(Normal, LightDir));
    
    // reflection vector
    float3 reflect = normalize(2 * diffuse * Normal - LightDir);
    
    // specular
    float specular = pow(saturate(dot(reflect, ViewDir)), 3);
    
   
  return 0.2 * Color + Color * diffuse + specular;
}

technique BumpShader
{
    pass P0
    {          
        Sampler[0] = (MainTextureSampler);		
        Sampler[1] = (BumpMapSampler);		
    
        VertexShader = compile vs_1_1 BumpVS();
        PixelShader  = compile ps_2_0 BumpPS();
    }

}


technique ParallaxBumpShader
{
    pass P0
    {          
        Sampler[0] = (MainTextureSampler);		
        Sampler[1] = (BumpMapSampler);		
        Sampler[1] = (HeightMapSampler);		
    
        VertexShader = compile vs_1_1 BumpVS();
        PixelShader  = compile ps_2_0 ParallaxPS();
    }

}

